@@ -17,15 +17,6 @@
 # You should have received a copy of the GNU General Public License
 # along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
 #
-# Notwithstanding any rights to use the Software granted by the foregoing,
-# if entities or individuals have received a Cease & Desist letter from
-# the Volatility Project, the Volatility Foundation, or its copyright holders
-# for violating the terms of the GPL version 2, those entities (their employees,
-# subcontractors, independent contractors, and affiliates) and / or persons
-# are granted no such rights and any use by any one or more of them is
-# expressly prohibited, in accordance with Section 4 of the GPL version 2.
-# Any rights granted to such entities and / or persons by earlier license
-# agreements have been previously terminated as to them.
 
 #pylint: disable-msg=C0111
 
@@ -311,6 +302,11 @@ class _SHARED_CACHE_MAP(obj.CType):
         FileSize = self.FileSize.QuadPart
         ValidDataLength = self.ValidDataLength.QuadPart
         SectionSize = self.SectionSize.QuadPart
+       
+        # Corrupted values: Win2003SP0x86.vmem 
+        if FileSize <= 0 or ValidDataLength <= 0:
+            return False
+
         #print "SectionSize 0x%x < 0 or FileSize < 0x%x ValidDataLength 0x%x"%(SectionSize,FileSize,ValidDataLength)
         #if SectionSize < 0 or (FileSize < ValidDataLength):
         if SectionSize < 0 or ((FileSize < ValidDataLength) and (ValidDataLength != 0x7fffffffffffffff)):
@@ -676,8 +672,8 @@ class DumpFiles(common.AbstractWindowsCommand):
                       help = 'Dump files for Process with physical address OFFSET',
                       action = 'store', type = 'int')
         config.add_option('PHYSOFFSET', short_option = 'Q', default = None,
-                      help = 'Dump File Object at physical address PHYSOFFSET',
-                      action = 'store', type = 'int')
+                      help = 'Dump File Object at physical address PHYSOFFSETs (comma delimited)',
+                      action = 'store', type = 'str')
         config.add_option('DUMP-DIR', short_option = 'D', default = None,
                       cache_invalidator = False,
                       help = 'Directory in which to dump extracted files')
@@ -697,7 +693,7 @@ class DumpFiles(common.AbstractWindowsCommand):
         # Possible filters include:
         # SharedCacheMap,DataSectionObject,ImageSectionObject,HandleTable,VAD
         config.add_option("FILTER", short_option = 'F', default = None,
-                            help = 'Filters to apply (comma-separated)')
+                            help = 'Filters to apply (comma-separated). Possible values:\n\nSharedCacheMap,DataSectionObject,ImageSectionObject,HandleTable,VAD')
 
     def filter_tasks(self, tasks):
         """ Reduce the tasks based on the user selectable PIDS parameter.
@@ -813,9 +809,14 @@ class DumpFiles(common.AbstractWindowsCommand):
         # are found with filescan that are not associated with a process
         # For example, $Mft.
         if self._config.PHYSOFFSET:
-            file_obj = obj.Object("_FILE_OBJECT", self._config.PHYSOFFSET, self.kaddr_space.base, native_vm = self.kaddr_space)
-            procfiles.append((None, [file_obj]))
-            #return
+            try:
+                phys = []
+                for p in self._config.PHYSOFFSET.split(","):
+                    file_obj = obj.Object("_FILE_OBJECT", int(p, 16), self.kaddr_space.base, native_vm = self.kaddr_space)
+                    phys.append(file_obj)
+                procfiles.append((None, phys))
+            except ValueError:
+                debug.error("Invalid PHYSOFFSET {0}".format(self._config.PHYSOFFSET))
 
         # Iterate through the process list and collect all references to
         # FILE_OBJECTS from both the VAD and HandleTable. Each open handle to a file
@@ -880,10 +881,13 @@ class DumpFiles(common.AbstractWindowsCommand):
         for pid, allfiles in procfiles:
             for file_obj in allfiles:
 
-                if not self._config.PHYSOFFSET:
-                    offset = file_obj.obj_offset
-                else:
-                    offset = self._config.PHYSOFFSET
+                # XXX TODO: remove these comments when accepted
+                #if not self._config.PHYSOFFSET:
+                offset = file_obj.obj_offset
+                #else:
+                # I'm not sure why we need to specify PHYSOFFSET here, 
+                # shouldn't we have a valid _FILE_OBJECT?
+                #    offset = self._config.PHYSOFFSET
 
                 name = None
 
@@ -1073,6 +1077,7 @@ class DumpFiles(common.AbstractWindowsCommand):
 
                 if self._config.SUMMARY_FILE:
                     json.dump(summaryinfo, summaryfo)
+                    summaryfo.write("\n")
                 of.close()
 
             elif summaryinfo['type'] == "ImageSectionObject":
@@ -1108,6 +1113,7 @@ class DumpFiles(common.AbstractWindowsCommand):
 
                 if self._config.SUMMARY_FILE:
                     json.dump(summaryinfo, summaryfo)
+                    summaryfo.write("\n")
                 of.close()
 
             elif summaryinfo['type'] == "SharedCacheMap":
@@ -1131,6 +1137,7 @@ class DumpFiles(common.AbstractWindowsCommand):
 
                 if self._config.SUMMARY_FILE:
                     json.dump(summaryinfo, summaryfo)
+                    summaryfo.write("\n")
                 of.close()
 
             else:
