@@ -5,10 +5,9 @@
 # This file is part of Volatility.
 #
 # Volatility is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License Version 2 as
-# published by the Free Software Foundation.  You may not use, modify or
-# distribute this program under any other version of the GNU General
-# Public License.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # Volatility is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -44,6 +43,12 @@ SERVICE_STATE_ENUM = {
             5: 'SERVICE_CONTINUE_PENDING',
             6: 'SERVICE_PAUSE_PENDING',
             7: 'SERVICE_PAUSED'}
+SERVICE_START_ENUM = {
+            0: 'SERVICE_BOOT_START',
+            1: 'SERVICE_SYSTEM_START',
+            2: 'SERVICE_AUTO_START',
+            3: 'SERVICE_DEMAND_START',
+            4: 'SERVICE_DISABLED'}
 
 svcscan_base_x86 = {
     '_SERVICE_HEADER': [ None, {
@@ -64,6 +69,7 @@ svcscan_base_x86 = {
     'ServiceProcess' : [ 0x24, ['pointer', ['_SERVICE_PROCESS']]],
     'Type' : [ 0x28, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
     'State' : [ 0x2c, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
+    'Start' : [ 0x44, ['Enumeration', dict(target = 'long', choices = SERVICE_START_ENUM)]],
     } ],
     '_SERVICE_PROCESS' : [ None, {
     'BinaryPath' : [ 0x8, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
@@ -90,6 +96,7 @@ svcscan_base_x64 = {
     'ServiceProcess' : [ 0x30, ['pointer', ['_SERVICE_PROCESS']]],
     'Type' : [ 0x38, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
     'State' : [ 0x3C, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
+    'Start' : [ 0x54, ['Enumeration', dict(target = 'long', choices = SERVICE_START_ENUM)]],
     } ],
     '_SERVICE_PROCESS': [ None, {
     'BinaryPath': [ 0x10, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
@@ -216,7 +223,8 @@ class ServiceVistax86(obj.ProfileModification):
 
     before = ['WindowsOverlay', 'WindowsObjectClasses', 'ServiceBase']
     conditions = {'os': lambda x: x == 'windows',
-                  'major': lambda x: x >= 6,
+                  'major': lambda x: x == 6,
+                  'minor': lambda x: x < 2,
                   'memory_model': lambda x: x == '32bit'}
 
     def modification(self, profile):
@@ -229,6 +237,7 @@ class ServiceVistax86(obj.ProfileModification):
             'DriverName': [ 0x1C, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
             'Type' : [ 0x20, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
             'State': [ 0x24, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
+            'Start' : [ 0x3C, ['Enumeration', dict(target = 'long', choices = SERVICE_START_ENUM)]],
             }]})
 
 class ServiceVistax64(obj.ProfileModification):
@@ -236,7 +245,8 @@ class ServiceVistax64(obj.ProfileModification):
 
     before = ['WindowsOverlay', 'WindowsObjectClasses', 'ServiceBase']
     conditions = {'os': lambda x: x == 'windows',
-                  'major': lambda x: x >= 6,
+                  'major': lambda x: x == 6,
+                  'minor': lambda x: x < 2,
                   'memory_model': lambda x: x == '64bit'}
 
     def modification(self, profile):
@@ -249,8 +259,67 @@ class ServiceVistax64(obj.ProfileModification):
             'DriverName': [ 0x28, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
             'Type' : [ 0x30, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
             'State': [ 0x34, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
+            'Start' : [ 0x4C, ['Enumeration', dict(target = 'long', choices = SERVICE_START_ENUM)]],
             }]})
 
+class Service8x64(obj.ProfileModification):
+    """Service structures for Win8/8.1 and Server2012/R2 64-bit"""
+
+    before = ['WindowsOverlay', 'WindowsObjectClasses', 'ServiceBase', 'ServiceVista']
+    conditions = {'os': lambda x: x == 'windows',
+                  'major': lambda x: x == 6,
+                  'minor': lambda x: x >= 2,
+                  'memory_model': lambda x: x == '64bit'}
+
+    def modification(self, profile):
+        profile.merge_overlay({
+            '_SERVICE_RECORD' : [ None, {
+            'Tag' : [ 0x0, ['String', dict(length = 4)]], 
+            'PrevEntry': [ 0x8, ['pointer', ['_SERVICE_RECORD']]],
+            'ServiceName' : [ 0x10, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
+            'DisplayName' : [ 0x18, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
+            'Order' : [ 0x20, ['unsigned int']],
+            'DriverName' : [ 0x38, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
+            'ServiceProcess' : [ 0x38, ['pointer', ['_SERVICE_PROCESS']]],
+            'Type' : [ 0x40, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
+            'State' : [ 0x44, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
+            'Start' : [ 0x5C, ['Enumeration', dict(target = 'long', choices = SERVICE_START_ENUM)]],
+            } ],
+            '_SERVICE_PROCESS': [ None, {
+            'BinaryPath': [ 0x18, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
+            'ProcessId': [ 0x28, ['unsigned int']],
+            } ],
+        })
+
+class Service8x86(obj.ProfileModification):
+    """Service structures for Win8/8.1 32-bit"""
+
+    before = ['WindowsOverlay', 'WindowsObjectClasses', 'ServiceBase', 'ServiceVista']
+    conditions = {'os': lambda x: x == 'windows',
+                  'major': lambda x: x == 6,
+                  'minor': lambda x: x >= 2,
+                  'memory_model': lambda x: x == '32bit'}
+
+    def modification(self, profile):
+        profile.vtypes.update({
+            '_SERVICE_RECORD' : [ None, {
+            'Tag' : [ 0x0, ['String', dict(length = 4)]], 
+            'PrevEntry': [ 0x4, ['pointer', ['_SERVICE_RECORD']]],
+            'ServiceName' : [ 0x8, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
+            'DisplayName' : [ 0xc, ['pointer', ['String', dict(encoding = 'utf16', length = 512)]]],
+            'Order' : [ 0x10, ['unsigned int']],
+            'DriverName' : [ 0x24, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
+            'ServiceProcess' : [ 0x24, ['pointer', ['_SERVICE_PROCESS']]],
+            'Type' : [ 0x28, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
+            'State' : [ 0x2c, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
+            'Start' : [ 0x44, ['Enumeration', dict(target = 'long', choices = SERVICE_START_ENUM)]],
+            } ],
+            '_SERVICE_PROCESS': [ None, {
+            'BinaryPath': [ 0xc, ['pointer', ['String', dict(encoding = 'utf16', length = 256)]]],
+            'ProcessId': [ 0x10, ['unsigned int']],
+            } ],
+        })
+
 #--------------------------------------------------------------------------------
 # svcscan plugin 
 #--------------------------------------------------------------------------------
@@ -353,15 +422,30 @@ class SvcScan(common.AbstractWindowsCommand):
             outfd.write(link)
         outfd.write("}\n")
 
+    @staticmethod
+    def get_service_dlls(regapi):
+        ccs = regapi.reg_get_currentcontrolset()
+        key_name = "{0}\\services".format(ccs)
+        dlls = {}
+        for subkey in regapi.reg_get_all_subkeys(hive_name = "system", key = key_name):
+            for rootkey in regapi.reg_get_all_subkeys(hive_name = "system", key = "", given_root = subkey):
+                if rootkey.Name == "Parameters":
+                    service_dll = regapi.reg_get_value(hive_name = "system", key = "", value = "ServiceDll", given_root = rootkey)
+                    if service_dll != None:
+                        dlls[utils.remove_unprintable(str(subkey.Name))] = "{0}".format(utils.remove_unprintable(service_dll))
+        return dlls
+
     def render_text(self, outfd, data):
 
         if self._config.VERBOSE:
             regapi = registryapi.RegistryApi(self._config)
+            dlls = self.get_service_dlls(regapi)
 
         for rec in data:
             # This can't possibly look neat in a table with columns...
             outfd.write("Offset: {0:#x}\n".format(rec.obj_offset))
             outfd.write("Order: {0}\n".format(rec.Order))
+            outfd.write("Start: {0}\n".format(rec.Start))
             outfd.write("Process ID: {0}\n".format(rec.Pid))
             outfd.write("Service Name: {0}\n".format(rec.ServiceName.dereference()))
             outfd.write("Display Name: {0}\n".format(rec.DisplayName.dereference()))
@@ -370,11 +454,7 @@ class SvcScan(common.AbstractWindowsCommand):
             outfd.write("Binary Path: {0}\n".format(rec.Binary))
 
             if self._config.VERBOSE:
-                ccs = regapi.reg_get_currentcontrolset()
-                val = regapi.reg_get_value(
-                                hive_name = "system", 
-                                key = "{0}\\services\\{1}\\Parameters".format(ccs, rec.ServiceName.dereference()), 
-                                value = "ServiceDll")
+                val = dlls.get("{0}".format(rec.ServiceName.dereference()), None)
                 if val is not None:
                     outfd.write("ServiceDll: {0}\n".format(val))
 
