@@ -5,10 +5,9 @@
 # This file is part of Volatility.
 #
 # Volatility is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License Version 2 as
-# published by the Free Software Foundation.  You may not use, modify or
-# distribute this program under any other version of the GNU General
-# Public License.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # Volatility is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,6 +40,70 @@ x64_native_types = copy.deepcopy(native_types.x64_native_types)
 x64_native_types['long'] = [8, '<q']
 x64_native_types['unsigned long'] = [8, '<Q']
 
+dyld_vtypes = {
+    'dyld32_image_info' : [12, {
+         'imageLoadAddress' : [0, ['pointer', ['unsigned int']]],
+         'imageFilePath'    : [4, ['pointer', ['char']]],
+         'imageFileModDate' : [8, ['pointer', ['unsigned int']]],  
+         }],
+    
+    'dyld32_all_image_infos' : [20 , {
+        'version'           : [0, ['unsigned int']],
+        'infoArrayCount'    : [4, ['unsigned int']],
+        'infoArray'         : [8, ['pointer', ['dyld32_image_info']]],
+        'notification'      : [12, ['pointer', ['void']]],
+        'processDetachedFromSharedRegion': [16, ['unsigned int']],
+    }],
+
+    'dyld64_image_info' : [24, {
+         'imageLoadAddress' : [0, ['pointer', ['unsigned int']]],
+         'imageFilePath'    : [8, ['pointer', ['char']]],
+         'imageFileModDate' : [16, ['pointer', ['unsigned int']]],  
+         }],
+    
+    'dyld64_all_image_infos' : [28 , {
+        'version'           : [0,  ['unsigned int']],
+        'infoArrayCount'    : [4,  ['unsigned int']],
+        'infoArray'         : [8,  ['pointer', ['dyld64_image_info']]],
+        'notification'      : [16, ['pointer', ['void']]],
+        'processDetachedFromSharedRegion': [24, ['unsigned int']],
+    }],
+}
+
+class DyldTypes(obj.ProfileModification):
+    conditions = {"os" : lambda x : x in ["mac"]}
+
+    def modification(self, profile):
+        profile.vtypes.update(dyld_vtypes)
+
+mig_vtypes_32 = {
+    'mig_hash_entry' : [16, {
+         'num'       : [0, ['int']],
+         'routine'   : [4, ['pointer', ['void']]],
+         'size'      : [8, ['int']],  
+         'callcount' : [12, ['unsigned int']],  
+         }],
+}
+
+mig_vtypes_64 = {
+    'mig_hash_entry' : [24, {
+         'num'       : [0, ['long long']],
+         'routine'   : [8, ['pointer', ['void']]],
+         'size'      : [16, ['int']],  
+         'callcount' : [20, ['unsigned int']],  
+         }],
+}
+
+class MigTypes(obj.ProfileModification):
+    conditions = {"os" : lambda x : x in ["mac"]}
+
+    def modification(self, profile):
+        if profile.metadata.get('memory_model', '32bit') == "32bit":
+            profile.vtypes.update(mig_vtypes_32)
+        else:
+            profile.vtypes.update(mig_vtypes_64)
+
+
 class catfishScan(scan.BaseScanner):
     """ Scanner for Catfish string for Mountain Lion """
     checks = []
@@ -126,13 +189,19 @@ class VolatilityMacIntelValidAS(obj.VolatilityMagic):
 
         string = self.obj_vm.read(version_addr, 60)
 
-        if string.startswith("Darwin"):
+        if string and string.startswith("Darwin"):
             self._set_profile_metadata(string)
             yield True
         else:
             yield False
 
 class vnode(obj.CType):
+    def is_dir(self):
+        return self.v_type == 2
+
+    def is_reg(self):
+        return self.v_type == 1
+
     def _do_calc_path(self, ret, vnodeobj, vname):
         if vnodeobj == None:
             return 
@@ -164,8 +233,55 @@ class vnode(obj.CType):
                 ret = "/" + ret
 
         return ret
+
+    def get_contents(self):
+        moc  = self.v_un.vu_ubcinfo.ui_control.moc_object
+        memq = moc.memq 
+
+        cur = memq.m("next").dereference_as("vm_page")
+
+        file_size = self.v_un.vu_ubcinfo.ui_size
+        
+        phys_as = utils.load_as(self.obj_vm.get_config(), astype = 'physical')
+        
+        while cur and cur.is_valid() and cur.offset < file_size:
+            buf = phys_as.zread(cur.phys_page * 4096, 4096)              
+
+            yield (cur.offset.v(), buf)
+ 
+            cur = cur.listq.next.dereference_as("vm_page")
+
+class fileglob(obj.CType):
+    
+    @property
+    def fg_type(self):
+        ret = self.members.get("fg_type")
+        if ret:
+            ret = self.m("fg_type")
+        else:    
+            ret = self.fg_ops.fo_type
+
+        ret = str(ret)
+        return ret
         
 class proc(obj.CType):   
+    def __init__(self, theType, offset, vm, name = None, **kwargs):
+        self.pack_fmt  = ""
+        self.pack_size = 0
+        self.addr_type = ""
+ 
+        obj.CType.__init__(self, theType, offset, vm, name, **kwargs)
+        
+        bit_string = str(self.task.map.pmap.pm_task_map or '')[9:]        
+        if bit_string.find("64BIT") == -1:
+            self.pack_fmt = "<I"
+            self.pack_size = 4
+            self.addr_type = "unsigned int"
+        else:
+            self.pack_fmt = "<Q"
+            self.pack_size = 8
+            self.addr_type = "unsigned long long"
+        
     @property
     def p_gid(self):
         cred = self.p_ucred
@@ -261,9 +377,33 @@ class proc(obj.CType):
         dt = obj.Object("UnixTimeStamp", offset = 0, vm = bufferas, is_utc = True)
 
         return dt
+    
+    def get_dyld_maps(self):        
+        proc_as = self.get_process_address_space()
+
+        if self.pack_size == 4:
+            dtype = "dyld32_all_image_infos"
+            itype = "dyld32_image_info"
+        else:
+            dtype = "dyld64_all_image_infos"
+            itype = "dyld64_image_info"
+
+        infos = obj.Object(dtype, offset=self.task.all_image_info_addr, vm=proc_as)
+
+        # the pointer address
+        info_buf = proc_as.read(infos.infoArray.obj_offset, self.pack_size)
+        if not info_buf:
+            return
+
+        info_addr = struct.unpack(self.pack_fmt, info_buf)[0] 
+
+        img_infos = obj.Object(theType = "Array", targetType = itype, offset = info_addr, count = infos.infoArrayCount, vm = proc_as)
+        
+        for info_addr in img_infos:
+            yield info_addr
+            #yield obj.Object("dyld_image_info", offset = info_addr, vm = proc_as)   
 
     def get_proc_maps(self):
-
         map = self.task.map.hdr.links.next
 
         for i in xrange(self.task.map.hdr.nentries):
@@ -299,6 +439,37 @@ class proc(obj.CType):
                         yield offset + hit
                 offset += min(to_read, scan_blk_sz)
 
+
+    def search_process_memory_rw_nofile(self, s):
+        """Search process memory. 
+
+        @param s: a list of strings like ["one", "two"]
+        """
+
+        # Allow for some overlap in case objects are 
+        # right on page boundaries 
+        overlap = 1024
+
+        scan_blk_sz = 1024 * 1024 * 10
+        addr_space = self.get_process_address_space()
+
+        for vma in self.get_proc_maps():
+            if vma.get_perms() != "rw-" or vma.get_path() != "":
+                continue
+
+            offset = vma.links.start
+            out_of_range = vma.links.start + (vma.links.end - vma.links.start)
+            while offset < out_of_range:
+                # Read some data and match it.
+                to_read = min(scan_blk_sz + overlap, out_of_range - offset)
+                data = addr_space.zread(offset, to_read)
+                if not data:
+                    break
+                for x in s:
+                    for hit in utils.iterfind(data, x):
+                        yield offset + hit
+                offset += min(to_read, scan_blk_sz)
+
     def get_arguments(self):
         proc_as = self.get_process_address_space()
 
@@ -365,16 +536,24 @@ class rtentry(obj.CType):
     @property
     def rx(self):
         if hasattr(self, "rt_stats"):
-            ret = self.rt_expire
+            ret = self.rt_stats.nstat_rxpackets 
         else:
             ret = "N/A"
 
     @property
     def delta(self):
-        if self.rt_expire == 0:
+        if self.expire() == 0:
             ret = 0
         else:
-            ret = self.rt_expire - self.base_uptime
+            ret = self.expire() - self.base_uptime
+
+        return ret
+
+    def expire(self):
+        if hasattr(self, "rt_expire"):
+            ret = self.rt_expire
+        else:
+            ret = 0
 
         return ret
 
@@ -506,15 +685,23 @@ class sysctl_oid(obj.CType):
             return "INVALID -1"
 
 class OSString(obj.CType):
-
     def __str__(self):
+        if self.string == 0:
+            return ""
+
         string_object = obj.Object("String", offset = self.string, vm = self.obj_vm, length = self.length)
         return str(string_object or '')
 
 class vm_map_entry(obj.CType):
+    @property
+    def start(self):
+        return self.links.start
+
+    @property
+    def end(self):
+        return self.links.end
 
     def get_perms(self):
-
         permask = "rwx"
         perms = ""
 
@@ -526,9 +713,27 @@ class vm_map_entry(obj.CType):
 
         return perms
 
-    def get_path(self):
 
-        vnode = self._get_vnode()
+    # used to find heap, stack, etc.
+    def get_special_path(self):
+        # check the heap
+        ret = ""
+
+        for i in [1, 2, 3, 4, 6, 7, 8, 9]:
+            if self.alias == i:
+                ret = "[heap]"
+                break
+
+        if ret != "":
+            return ret
+
+        if self.alias == 30:
+            ret = "[stack]"
+
+        return ret 
+
+    def get_path(self):
+        vnode = self.get_vnode()
     
         if type(vnode) == str and vnode == "sub_map":
             ret = vnode  
@@ -544,13 +749,14 @@ class vm_map_entry(obj.CType):
                 
         return ret
 
-    def _get_vnode(self):
+    def get_vnode(self):
+        map_obj = self
 
         if self.is_sub_map == 1:
-            return "sub_map" 
+            return "sub_map"
 
         # find_vnode_object
-        vnode_object = self.object.vm_object 
+        vnode_object = map_obj.object.vm_object 
 
         while vnode_object.shadow.dereference() != None:
             vnode_object = vnode_object.shadow.dereference()
@@ -565,6 +771,76 @@ class vm_map_entry(obj.CType):
 
         return ret
 
+class inpcb(obj.CType):
+    
+    def get_tcp_state(self):
+        tcp_states = (
+              "CLOSED",
+              "LISTEN",
+              "SYN_SENT",
+              "SYN_RECV",
+              "ESTABLISHED",
+              "CLOSE_WAIT",
+              "FIN_WAIT1",
+              "CLOSING",
+              "LAST_ACK",
+              "FIN_WAIT2",
+              "TIME_WAIT")
+
+        tcpcb = self.inp_ppcb.dereference_as("tcpcb")
+
+        return tcp_states[tcpcb.t_state]
+
+    def ipv4_info(self):
+        lip = self.inp_dependladdr.inp46_local.ia46_addr4.s_addr.v()    
+        lport = self.inp_lport 
+
+        rip = self.inp_dependfaddr.inp46_foreign.ia46_addr4.s_addr.v()
+        rport = self.inp_fport 
+    
+        return [lip, lport, rip, rport]
+
+    def ipv6_info(self):
+        lip = self.inp_dependladdr.inp6_local.__u6_addr.v()
+        lport = self.inp_lport 
+
+        rip = self.inp_dependfaddr.inp6_foreign.__u6_addr.v() 
+        rport = self.inp_fport 
+
+        return [lip, lport, rip, rport]
+
+class inpcbinfo(obj.CType):
+    @property
+    def hashbase(self):
+        ret = self.members.get("hashbase")
+        if ret is None:
+            ret = self.ipi_hashbase
+        else:
+            ret = self.m("hashbase")
+
+        return ret
+
+    @property
+    def hashmask(self):
+        ret = self.members.get("hashmask")
+        if ret is None:
+            ret = self.ipi_hashmask
+        else:
+            ret = self.m("hashmask")
+
+        return ret
+
+    @property
+    def listhead(self):
+        ret = self.members.get("listhead")
+        if ret is None:
+            ret = self.ipi_listhead
+        else:
+            ret = self.m("listhead")
+
+        return ret
+
+
 class socket(obj.CType):
     @property
     def family(self):
@@ -611,36 +887,17 @@ class socket(obj.CType):
         
         return ret
         
-    def _parse_ipv4(self, pcb):
-        lip = pcb.inp_dependladdr.inp46_local.ia46_addr4.s_addr.v()    
-        lport = pcb.inp_lport 
-
-        rip = pcb.inp_dependfaddr.inp46_foreign.ia46_addr4.s_addr.v()
-        rport = pcb.inp_fport 
-    
-        return [lip, lport, rip, rport]
-
-    def _parse_ipv6(self, pcb):
-        lip = pcb.inp_dependladdr.inp6_local.__u6_addr.v()
-        lport = pcb.inp_lport 
-
-        rip = pcb.inp_dependfaddr.inp6_foreign.__u6_addr.v() 
-        rport = pcb.inp_fport 
-
-        return [lip, lport, rip, rport]
-
     def get_connection_info(self):
         ipcb = self.so_pcb.dereference_as("inpcb")
         
         if self.family == 2:
-            ret = self._parse_ipv4(ipcb)
+            ret = ipcb.ipv4_info()
         else:
-            ret = self._parse_ipv6(ipcb)
+            ret = ipcb.ipv6_info()
 
         return ret
 
 class sockaddr_dl(obj.CType):
-
     def v(self):
         """Get the value of the sockaddr_dl object."""
 
@@ -660,9 +917,7 @@ class sockaddr_dl(obj.CType):
         return ret
 
 class sockaddr(obj.CType):
-    
     def get_address(self):
-
         family = self.sa_family
 
         ip = ""
@@ -681,6 +936,70 @@ class sockaddr(obj.CType):
 
         return ip
 
+class dyld32_image_info(obj.CType):
+    def _read_ptr(self, addr):
+        addr = self.obj_vm.read(addr, 4)
+        if not addr:
+            ret = None
+        else:
+            ret = struct.unpack("<I", addr)[0]         
+        return ret
+
+    @property
+    def imageFilePath(self):
+        addr = self.m("imageFilePath").obj_offset
+        addr = self._read_ptr(addr)
+        
+        if addr == None:
+            return ""
+
+        buf = self.obj_vm.read(addr, 256)
+        if buf:
+            idx = buf.find("\x00")
+            if idx != -1:
+                buf = buf[:idx]
+
+        return buf
+
+    @property
+    def imageLoadAddress(self):
+        addr = self.m("imageLoadAddress").obj_offset
+        addr = self._read_ptr(addr)
+                 
+        return addr
+
+class dyld64_image_info(obj.CType):
+    def _read_ptr(self, addr):
+        addr = self.obj_vm.read(addr, 8)
+        if addr == None:
+            ret = None
+        else:
+            ret = struct.unpack("<Q", addr)[0]         
+        return ret
+
+    @property
+    def imageFilePath(self):
+        addr = self.m("imageFilePath").obj_offset
+        addr = self._read_ptr(addr)
+
+        if addr == None:
+            return ""
+
+        buf = self.obj_vm.read(addr, 256)
+        if buf:
+            idx = buf.find("\x00")
+            if idx != -1:
+                buf = buf[:idx]
+
+        return buf
+
+    @property
+    def imageLoadAddress(self):
+        addr = self.m("imageLoadAddress").obj_offset
+        addr = self._read_ptr(addr)
+                 
+        return addr
+
 def exec_vtypes(filename):
     env = {}
     exec(filename, dict(__builtins__ = None), env)
@@ -693,36 +1012,39 @@ def parse_dsymutil(data, module):
 
     want_lower = ["_IdlePML4"]        
 
+    type_map = {}
+    type_map[module] = {}
+
     arch = ""
 
     # get the system map
     for line in data.splitlines():
         ents = line.split()
 
-        match = re.search("\[.*?\)\s+[0-9A-Fa-z]+\s+\d+\s+([0-9A-Fa-f]+)\s'(\w+)'", line)
+        match = re.search("\[.*?\(([^\)]+)\)\s+[0-9A-Fa-z]+\s+\d+\s+([0-9A-Fa-f]+)\s'(\w+)'", line)
 
         if match:
-            (addr, name) = match.groups()
-
+            (sym_type, addr, name) = match.groups()
+            sym_type = sym_type.strip()
+    
             addr = int(addr, 16)
 
-            if addr == 0:
+            if addr == 0 or name == "":
                 continue
 
             if not name in sys_map[module]:
-                sys_map[module][name] = [(0, "default value")]
-
+                sys_map[module][name] = [(addr, sym_type)]
+                
             # every symbol is in the symbol table twice
             # except for the entries in 'want_lower', we need the higher address for all 
-            if name in sys_map[module]:
-                oldaddr = sys_map[module][name][0][0]
-    
-                if oldaddr > addr and name not in want_lower:
-                    pass
-                else:
-                    sys_map[module][name] = [(addr, "sym type?")]
-            else:
-                sys_map[module][name] = [(addr, "sym type?")]
+            oldaddr = sys_map[module][name][0][0]
+            if addr < oldaddr and name in want_lower:
+                sys_map[module][name] = [(addr, sym_type)]
+        
+            if not addr in type_map[module]:
+                type_map[module][addr] = (name, [sym_type])
+
+            type_map[module][addr][1].append(sym_type)
 
         elif line.find("Symbol table for") != -1:
             if line.find("i386") != -1:
@@ -733,23 +1055,25 @@ def parse_dsymutil(data, module):
     if arch == "":
         return None
 
-    return arch, sys_map
+    return arch, sys_map, type_map
 
 def MacProfileFactory(profpkg):
 
     vtypesvar = {}
     sysmapvar = {}
+    typesmapvar = {}
 
     memmodel, arch = "32bit", "x86"
     profilename = os.path.splitext(os.path.basename(profpkg.filename))[0]
  
     for f in profpkg.filelist:
         if 'symbol.dsymutil' in f.filename.lower():
-            memmodel, sysmap = parse_dsymutil(profpkg.read(f.filename), "kernel")
+            memmodel, sysmap, typemap = parse_dsymutil(profpkg.read(f.filename), "kernel")
             if memmodel == "64bit":
                 arch = "x64"
             
             sysmapvar.update(sysmap)
+            typesmapvar.update(typemap)
             debug.debug("{2}: Found system file {0} with {1} symbols".format(f.filename, len(sysmapvar.keys()), profilename))
 
         elif f.filename.endswith(".vtypes"):
@@ -770,13 +1094,19 @@ def MacProfileFactory(profpkg):
 
 
         def __init__(self, *args, **kwargs):
-            self.sys_map = {}
-            self.shift_address = 0
+            self._init_vars()
             obj.Profile.__init__(self, *args, **kwargs)
 
+        def _init_vars(self):
+            self.sys_map = {}
+            self.type_map = {}
+            self.shift_address = 0
+            self.sba_cache = {}
+            self.sbat_cache = {}
+            
         def clear(self):
             """Clear out the system map, and everything else"""
-            self.sys_map = {}
+            self._init_vars() 
             obj.Profile.clear(self)
 
         def reset(self):
@@ -797,6 +1127,7 @@ def MacProfileFactory(profpkg):
         def load_sysmap(self):
             """Loads up the system map data"""
             self.sys_map.update(sysmapvar)
+            self.type_map.update(typesmapvar)
 
         # Returns a list of (name, addr)
         def get_all_symbols(self, module = "kernel"):
@@ -815,7 +1146,7 @@ def MacProfileFactory(profpkg):
 
                     ret.append([name, addr])
             else:
-                debug.info("All symbols  requested for non-existent module %s" % module)
+                debug.info("All symbols requested for non-existent module %s" % module)
 
             return ret
 
@@ -832,19 +1163,94 @@ def MacProfileFactory(profpkg):
 
             return ret
 
-        def get_symbol_by_address(self, module, sym_address):
+        ############################################
+
+        # Returns a list of (name, addr)
+        def get_all_function_symbols(self, module = "kernel"):
+            """ Gets all the function tuples for the given module """
+            ret = []
+
+            symtable = self.type_map
+
+            if module in symtable:
+                mod = symtable[module]
+
+                for (addr, (name, _sym_types)) in mod.items():
+                    if self.shift_address and addr:
+                        addr = addr + self.shift_address
+
+                    ret.append([name, addr])
+            else:
+                debug.info("All symbols requested for non-existent module %s" % module)
+
+            return ret
+
+        def get_all_function_addresses(self, module = "kernel"):
+            """ Gets all the function addresses for the given module """
+            # returns a hash table for quick looks
+            # the main use of this function is to see if an address is known
+            ret = {}
+
+            symbols = self.get_all_function_symbols(module)
+
+            for (_name, addr) in symbols:
+                ret[addr] = 1
+
+            return ret
+
+        def _get_symbol_by_address_type(self, module, wanted_sym_address, wanted_sym_type):
             ret = ""
+            
+            symtable = self.type_map
+
+            mod = symtable[module]
+
+            for (addr, (name, sym_types)) in mod.items():
+                for sym_type in sym_types:
+                    key = "%s|%x|%s" % (module, addr, sym_type)
+                    self.sbat_cache[key] = name
+
+                    if (wanted_sym_address == addr or wanted_sym_address == self.shift_address + addr) and wanted_sym_type == sym_type:
+                        ret = name
+                        break
+ 
+            return ret
+        
+        def get_symbol_by_address_type(self, module, sym_address, sym_type):
+            key = "%s|%x|%s" % (module, sym_address, sym_type)
+            if key in self.sbat_cache:
+                ret = self.sbat_cache[key]
+            else:
+                ret = self._get_symbol_by_address_type(module, sym_address, sym_type)
+            
+            return ret
+
+
+        def _get_symbol_by_address(self, module, sym_address):
+            ret = ""
+            
             symtable = self.sys_map
 
             mod = symtable[module]
 
             for (name, addrs) in mod.items():
-
-                for (addr, addr_type) in addrs:
+                for (addr, _) in addrs:
+                    key = "%s|%x" % (module, addr)
+                    self.sba_cache[key] = name
+         
                     if sym_address == addr or sym_address == self.shift_address + addr:
                         ret = name
                         break
-
+                    
+            return ret
+        
+        def get_symbol_by_address(self, module, sym_address):
+            key = "%s|%x" % (module, sym_address)
+            if key in self.sba_cache:
+                ret = self.sba_cache[key]
+            else:
+                ret = self._get_symbol_by_address(module, sym_address)
+            
             return ret
 
         def get_all_symbol_names(self, module = "kernel"):
@@ -970,8 +1376,13 @@ class MacObjectClasses(obj.ProfileModification):
             'VolatilityDTB': VolatilityDTB,
             'VolatilityMacIntelValidAS' : VolatilityMacIntelValidAS,
             'proc'  : proc,
+            'dyld32_image_info' : dyld32_image_info,
+            'dyld64_image_info' : dyld64_image_info,
+            'fileglob' : fileglob,
             'vnode' : vnode,
             'socket' : socket,
+            'inpcbinfo' : inpcbinfo,
+            'inpcb' : inpcb,
             'zone' : zone,
             'OSString' : OSString,
             'OSString_class' : OSString,
@@ -1032,6 +1443,12 @@ mac_overlay = {
     'sysctl_oid' : [ None, { 
         'oid_name' : [ None, ['pointer', ['String', dict(length = 256)]]], 
         }], 
+    'dyld32_image_info' : [ None, { 
+        'imageFilePath' : [ None, ['pointer', ['String', dict(length = 256)]]], 
+        }], 
+    'dyld64_image_info' : [ None, { 
+        'imageFilePath' : [ None, ['pointer', ['String', dict(length = 256)]]], 
+        }], 
     'sockaddr_un': [ None, { 
         'sun_path' : [ None, ['String', dict(length = 104)]],
         }],
@@ -1047,3 +1464,4 @@ mac_overlay = {
         }], 
 }
 
+
