@@ -1,14 +1,13 @@
 # Volatility
 # Copyright (C) 2007-2013 Volatility Foundation
-# Copyright (c) 2012 Jamie Levy (Gleeda) <jamie.levy@gmail.com>
+# Copyright (c) 2012 Jamie Levy (Gleeda) <jamie@memoryanalysis.net>
 #
 # This file is part of Volatility.
 #
 # Volatility is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License Version 2 as
-# published by the Free Software Foundation.  You may not use, modify or
-# distribute this program under any other version of the GNU General
-# Public License.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # Volatility is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,7 +21,7 @@
 """
 @author:       Jamie Levy (Gleeda)
 @license:      GNU General Public License 2.0
-@contact:      jamie.levy@gmail.com
+@contact:      jamie@memoryanalysis.net
 @organization: Volatility Foundation
 """
 
@@ -36,7 +35,6 @@ import volatility.plugins.overlays.basic as basic
 import volatility.timefmt as timefmt
 import struct
 import datetime 
-import calendar
 
 '''
 Some references for further reading, all of which were used for building this plugin:
@@ -293,13 +291,14 @@ class ITEMPOS(obj.CType):
         return fileattrs
 
     def body(self, details):
-        return "0|[SHELLBAGS ITEMPOS] Name: {3}/Attrs: {4}/{5}|0|---------------|0|0|0|{0}|{1}|{2}|{2}\n".format(
+        return "0|[{6}SHELLBAGS ITEMPOS] Name: {3}/Attrs: {4}/{5}|0|---------------|0|0|0|{0}|{1}|{2}|{2}\n".format(
             self.Attributes.AccessDate.v(), 
             self.Attributes.ModifiedDate.v(),
             self.Attributes.CreatedDate.v(),
             str(self.Attributes.UnicodeFilename), 
             self.get_file_attrs(), 
-            details)
+            details,
+            self.obj_vm._config.MACHINE)
 
     def __str__(self):
         return "{0:<14} {1:30} {2:30} {3:30} {4:25} {5}".format(self.Attributes.FileName,
@@ -328,13 +327,14 @@ class FILE_ENTRY(ITEMPOS):
         return fileattrs
 
     def body(self, details):
-        return "0|[SHELLBAGS FILE_ENTRY] Name: {3}/Attrs: {4}/{5}|0|---------------|0|0|0|{0}|{1}|{2}|{2}\n".format(
+        return "0|[{6}SHELLBAGS FILE_ENTRY] Name: {3}/Attrs: {4}/{5}|0|---------------|0|0|0|{0}|{1}|{2}|{2}\n".format(
             self.Attributes.AccessDate.v(), 
             self.Attributes.ModifiedDate.v(),
             self.Attributes.CreatedDate.v(),
             str(self.Attributes.UnicodeFilename),
             self.get_file_attrs(),
-            details)
+            details,
+            self.obj_vm._config.MACHINE)
 
     def __str__(self):
         return "{0:<14} {1:30} {2:30} {3:30} {4:25}".format(self.Attributes.FileName,
@@ -487,71 +487,6 @@ class NETWORK_SHARE(NETWORK_VOLUME_NAME):
 #####  End Type Overrides #####
 
         
-class DosDate(obj.NativeType):
-    def __init__(self, theType, offset, vm, is_utc = False, **kwargs):
-        self.is_utc = is_utc
-        obj.NativeType.__init__(self, theType, offset, vm, format_string = "<I", **kwargs)
-
-    def as_dos_timestamp(self):
-        return obj.NativeType.v(self)
-
-    def v(self):
-        value = self.as_dos_timestamp()
-        return self.dos_to_unix_time(value)
-
-    def __nonzero__(self):
-        return self.v() != 0
-
-    def __str__(self):
-        return "{0}".format(self)
-
-    def as_datetime(self):
-        try:
-            dt = datetime.datetime.utcfromtimestamp(self.v())
-            if self.is_utc:
-                # Only do dt.replace when dealing with UTC
-                dt = dt.replace(tzinfo = timefmt.UTC())
-        except ValueError, e:
-            return obj.NoneObject("Datetime conversion failure: " + str(e))
-        return dt
-
-    def __format__(self, formatspec):
-        """Formats the datetime according to the timefmt module"""
-        dt = self.as_datetime()
-        if dt != None:
-            return format(timefmt.display_datetime(dt), formatspec)
-        return "-"
-
-    
-    def dos_to_unix_time(self, dosdate):
-        """  
-        Every previous conversion algorithm takes in two unsigned shorts separately.  
-        We're not doing that here, but instead getting those shorts from an unsigned int (dosdate)
-
-        dosdate: 4 bytes little endian converted to:
-        date: 2 bytes
-        time: 2 bytes
-
-        conversion to datetime taken from: http://code.google.com/p/libforensics/
-        dosdate is already in UTC: http://download.polytechnic.edu.na/pub4/download.sourceforge.net/pub/sourceforge/l/project/li/liblnk/Documentation/Windows%20Shell%20Item%20format/Windows%20Shell%20Item%20format.pdf
-        """
-        date = struct.unpack(">H", ''.join([chr(x) for x in [(dosdate >> 8) & 0xff, (dosdate & 0xff)]]))[0]
-        time = struct.unpack(">H", ''.join([chr(x) for x in [(dosdate >> 24) & 0xff, (dosdate >> 16) & 0xff]]))[0]
-        seconds = (time & 0x1F) * 2
-        minutes = (time & 0x7E0) >> 5
-        hours = (time & 0xF800) >> 11
-        day = date & 0x1F 
-        month = (date & 0x1E0) >> 5
-        year = ((date & 0xFE00) >> 9) + 1980
-
-        #convert into timestamp and return:
-        try:
-            return calendar.timegm(datetime.datetime(year, month, day, hours, minutes, seconds).utctimetuple())
-        except ValueError:
-            return 0
-        # if we use the following we need to s/utcfromtimestamp/fromtimestamp/ in as_datetime() function:
-        #return time.mktime(datetime.datetime(year, month, day, hours, minutes, seconds).timetuple())
-
 class NullString(basic.String):
     def __str__(self):
         result = self.obj_vm.zread(self.obj_offset, self.length).split("\x00\x00")[0].replace("\x00", "")
@@ -666,7 +601,6 @@ class ShellBagsTypesXP(obj.ProfileModification):
     def modification(self, profile):
         profile.object_classes.update({
             'NullString': NullString,
-            'DosDate':DosDate,
             '_GUID':_GUID,
             'ITEMPOS':ITEMPOS,
             'FILE_ENTRY':FILE_ENTRY,
@@ -722,7 +656,6 @@ class ShellBagsTypesVista(obj.ProfileModification):
     def modification(self, profile):
         profile.object_classes.update({
             'NullString': NullString,
-            'DosDate':DosDate,
             '_GUID':_GUID,
             'ITEMPOS':ITEMPOS,
             'FILE_ENTRY':FILE_ENTRY,
@@ -775,11 +708,10 @@ class ShellBagsTypesWin7(obj.ProfileModification):
     before = ['WindowsObjectClasses']
     conditions = {'os': lambda x: x == 'windows',
                   'major': lambda x: x == 6, 
-                  'minor': lambda x: x == 1}
+                  'minor': lambda x: x >= 1}
     def modification(self, profile):
         profile.object_classes.update({
             'NullString': NullString,
-            'DosDate':DosDate,
             '_GUID':_GUID,
             'ITEMPOS':ITEMPOS,
             'FILE_ENTRY':FILE_ENTRY,
@@ -800,6 +732,8 @@ class ShellBags(common.AbstractWindowsCommand):
     """Prints ShellBags info"""
     def __init__(self, config, *args, **kwargs):
         common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)
+        config.add_option("MACHINE", default = "",
+                        help = "Machine name to add to timeline header")
         self.supported = ["FILE_ENTRY", "FOLDER_ENTRY", "CONTROL_PANEL", "VOLUME_NAME", "NETWORK_VOLUME_NAME", "NETWORK_SHARE", "UNKNOWN_00"]
         self.paths = {}
 
@@ -864,6 +798,8 @@ class ShellBags(common.AbstractWindowsCommand):
         version = (addr_space.profile.metadata.get('major', 0), 
                    addr_space.profile.metadata.get('minor', 0))
         
+        if self._config.MACHINE != "":
+            self._config.update("MACHINE", "{0} ".format(self._config.MACHINE))
         #set our current registry of interest and get its path
         regapi = registryapi.RegistryApi(self._config)
         regapi.reset_current()
@@ -880,7 +816,7 @@ class ShellBags(common.AbstractWindowsCommand):
                 keys = [(k, bk + "\\" + k.Name) for k in regapi.reg_get_all_subkeys("ntuser.dat", key = None, given_root = cat)]
                 for key, start in keys:
                     if key.Name:
-                        if str(key.Name).lower().find("cmi-create") != -1 or str(key.Name).find("S-") != -1 or seen.get(start + "\\" + k.Name, None) != None:
+                        if seen.get(start + "\\" + k.Name, None) != None:
                             continue
                         seen[start + "\\" + k.Name] = key.obj_offset
                         subkeys = [k for k in regapi.reg_get_all_subkeys("ntuser.dat", key = None, given_root = key)]
@@ -898,7 +834,7 @@ class ShellBags(common.AbstractWindowsCommand):
                     keys = [(k, bk + "\\" + k.Name) for k in regapi.reg_get_all_subkeys("UsrClass.dat", key = None, given_root = cat)]
                     for key, start in keys:
                         if key.Name:
-                            if str(key.Name).lower().find("cmi-create") != -1 or str(key.Name).find("S-") != -1 or seen.get(start + "\\" + k.Name, None) != None:
+                            if seen.get(start + "\\" + k.Name, None) != None:
                                 continue
                             seen[start + "\\" + k.Name] = key.obj_offset
                             subkeys = [k for k in regapi.reg_get_all_subkeys("UsrClass.dat", key = None, given_root = key)]
